function OUT = dependencemap(FD,varargin)


% downslope area for specific locations in a digital elevation model
%
% Syntax
%
%     I = influencemap(FD,L)
%     I = influencemap(FD,ix)
%     I = influencemap(FD,x,y)
%
% Description
%
%     influencemap returns a logical matrix masking the downslope part
%     of the digital elevation model that is drained by specified cells.
%
% Input
%
%     FD        flow direction object (FlowDirObj)
%     L         logical grid (GRIDobj)
%     ix        linear index into a grid
%     x,y       coordinate pairs
%
% Output
%
%     I         logical influence grid (GRIDobj)
%
% Example
%
%
% See also: FLOWobj, FLOWobj/DEPENDENCEMAP
%
% Author: Wolfgang Schwanghart (w.schwanghart[at]geo.uni-potsdam.de)
% Date: 5. January, 2013



%% check input arguments
narginchk(1,3)
if nargin == 2;
    % SEED pixels are either supplied as logical matrix, GRIDobj, or linear
    % index
    SEED = varargin{1};
    isGRIDobj = isa(SEED,'GRIDobj');   
    if (islogical(SEED) || isGRIDobj)
        validatealignment(FD,SEED);
        if isGRIDobj;
            SEED = SEED.Z;
        end
    else
        % SEED is supposed to be supplied as linear index in the GRIDobj
        ix   = varargin{1};
        SEED = false(FD.size);
        SEED(ix) = true;
    end
elseif nargin == 3;
    % SEED pixels are supplied as coordinate pairs
    ix   = coord2ind(FD,varargin{1},varargin{2});
    SEED = false(FD.size);
    SEED(ix) = true;
end


% this is a very crude and slow implementation of a graph traversal
% algorithm since all nodes are visited

if ~(exist(['dependencemap_mex.' mexext],'file') == 3);
    
    for r = numel(FD.ix):-1:1;
        SEED(FD.ix(r)) = SEED(FD.ix(r)) || SEED(FD.ixc(r));
    end
    
else
        
    SEED = dependencemap_mex(FD.ix,FD.ixc,FD.size,find(SEED));    
end


%% Prepare Output
% empty GRIDobj
OUT = copy2GRIDobj(FD);
% write output to GRIDobj
OUT.Z = SEED;
OUT.zunit = 'logical';
OUT.name  = 'dependence map';


end








