classdef GRIDobj
    
% Create instance of a GRIDobj
%
% Syntax
%
%     DEM = GRIDobj(X,Y,dem)
%     DEM = GRIDobj('ESRIasciiGrid.txt')
%     DEM = GRIDobj('GeoTiff.tif')
%     DEM = GRIDobj();
%     DEM = GRIDobj([]);
%
% Description
%
%     GRIDobj creates an instance of the grid class, which contains a
%     numerical or logical matrix and information on georeferencing. When a
%     GRIDobj is created from a file, the number format of the data in
%     GRIDobj is either single or double. Unsigned and signed integers are
%     converted to single. For unsigned integers, missing values are
%     assumed to be denoted as intmax(class(input)). For signed integers,
%     missing values are assumed to be intmin(class(input)). Please check,
%     that missing values in your data have been identified correctly
%     before further analysis.
%
%     Note that while throughout this help text GRIDobj is associated with
%     gridded digital elevation models, instances of GRIDobj can contain
%     other gridded, single band, datasets such as flow accumulation grids, 
%     gradient grids etc.
%
%     DEM = GRIDobj(X,Y,dem) creates a DEM object from the coordnate
%     matrices or vectors X and Y and the matrix dem. The elements of dem
%     refer to the elevation of each pixel. 
%
%     DEM = GRIDobj('ESRIasciiGrid.txt') creates a DEM object from an ESRI 
%     Ascii grid exported from other GI systems. 
%
%     DEM = GRIDobj('GeoTiff.tif') creates a DEM object from a Geotiff.
%
%     DEM = GRIDobj() opens a dialog box to read either an ESRI Ascii Grid
%     or a Geotiff.
%
%     DEM = GRIDobj([]) creates an empty instance of GRIDobj
%
% Example
%
%     % Load DEM
%     DEM = GRIDobj('srtm_bigtujunga30m_utm11.tif');
%     % Display DEM
%     imageschs(DEM)
%
% See also: FLOWobj, STREAMobj
%
% Author: Wolfgang Schwanghart (w.schwanghart[at]geo.uni-potsdam.de)
% Date: 17. June, 2013

    
    properties
        Z          % matrix with elevation values
        cellsize   % cellsize of the grid (scalar)
        refmat     % 3-by-2 affine transformation matrix (see makerefmat)
        size       % size of the .Z matrix (two element vector)
        name       % optional name
        zunit      % unit of the .Z matrix
        xyunit     % unit of the coordinates
        georef     % additional information on spatial referencing
        
    end
    
    methods
        function DEM = GRIDobj(varargin)
            
           
            
            if nargin == 3;
                
                %% GRIDobj is created from three matrices
                % GRIDobj(X,Y,dem)
                X = varargin{1};
                Y = varargin{2};
                
                if min(size(X)) > 1;
                    X = X(1,:);
                end
                if min(size(Y)) > 1;
                    Y = Y(:,1);
                end
                
                DEM.Z   = varargin{3};
                DEM.size = size(DEM.Z);
                
                if numel(X) ~= DEM.size(2) || numel(Y) ~= DEM.size(1)
                    error('TopoToolbox:GRIDobj',...
                          ['Coordinate matrices/vectors don''t fit the size of the \n'...
                           'the digital elevation model']);
                end
                
                if (Y(2)-Y(1)) > 0
                    % the y coordinate vector must be monotonically
                    % decreasing, so that the left upper edge of the DEM is
                    % north-west (on the northern hemisphere).
                    DEM.Z = flipud(DEM.Z);
                    Y = Y(end:-1:1);
                end
                
                dy = Y(2)-Y(1);
                dx = X(2)-X(1);
                
                if abs(abs(dx)-abs(dy))>1e-9;
                    error('TopoToolbox:GRIDobj',...
                          'The resolution in x- and y-direction must be the same');
                end
                    
                
                DEM.refmat = double([0 dy;...
                              dx 0;...
                              X(1)-dx Y(1)-dy]);
                
                DEM.cellsize = dx;
                DEM.georef = [];
                DEM.name   = [];
            
                
            elseif nargin <= 1;
                
                
                if nargin == 0
                    % No input arguments. File dialog box will open and ask
                    % for a txt or tiff file as input
                    FilterSpec  = {'*.txt';'*.asc';'*.tif';'*.tiff'};
                    DialogTitle = 'Select ESRI ASCII grid or GeoTiff';
                    [FileName,PathName] = uigetfile(FilterSpec,DialogTitle);
                
                    if FileName == 0;
                        error('TopoToolbox:incorrectinput',...
                                'no file was selected')
                    end
                
                    filename = fullfile(PathName, FileName);
                    
                elseif nargin == 1;
                    % One input argument
                    if isempty(varargin{1})
                        % if empty array than return empty GRIDobj
                        return
                    end
                
                    % GRIDobj is created from a file
                    filename = varargin{1};
                end
                
                % check if file exists
                if exist(filename,'file')~=2
                    error('File doesn''t exist')
                end
                    
                
                % separate filename into path, name and extension
                [pathstr,DEM.name,ext]  = fileparts(filename);
                

                if any(strcmpi(ext,{'.tif', '.tiff'}))
                    % it is a GeoTiff
                    try
                        % try to read using geotiffread (requires mapping
                        % toolbox)
                        [DEM.Z, DEM.refmat, ~] = geotiffread(filename);
                        DEM.georef = geotiffinfo(filename);
                    catch err
                        % if this doesn't work, try reading the world-file
                        
                        tfwfile = fullfile(pathstr,[DEM.name '.tfw']);
                        if exist(tfwfile,'file');
                            try 
                                % prefer builtin worldfileread, if
                                % available
                                DEM.refmat = worldfileread(tfwfile);
                            catch ME
                                W = dlmread(tfwfile);
                                
                                DEM.refmat(2,1) = W(1,1);
                                DEM.refmat(1,2) = W(4,1);
                                DEM.refmat(3,1) = W(5,1)-W(1);
                                DEM.refmat(3,2) = W(6,1)-W(4);
                            end
                                
                                
                            DEM.Z = imread(filename);
                        else
                            rethrow(err)
                        end
                    end
        
                else
                    % it is a ESRI ascii grid
                    [DEM.Z,DEM.refmat] = rasterread(filename);
                    DEM.georef = [];
                end
                DEM.size = size(DEM.Z);
                DEM.cellsize = abs(DEM.refmat(2));
                
                
                % remove nans
                demclass = class(DEM.Z);
                switch demclass
                    case {'uint8','uint16','uint32'}
                        % unsigned integer
                        DEM.Z = single(DEM.Z);
                        DEM.Z(DEM.Z==intmax(demclass)) = nan;
                    case {'int8','int16','int32'}
                        % signed integer
                        DEM.Z = single(DEM.Z);
                        DEM.Z(DEM.Z==intmin(demclass)) = nan;
                    case {'double','single'}
                    case 'logical'
                    otherwise
                        error('TopoToolbox:GRIDobj','unrecognized class')
                end
                
                
            end 
        end
    
    end
    
end
    



% Subfunction for ASCII GRID import
function [Z,refmat] = rasterread(file)

fid=fopen(file,'r');
% loop through header

header = struct('ncols',[],...
                'nrows',[],...
				'xllcorner',[],...
				'yllcorner',[],...
				'cellsize',[],...
				'nodata',[]);
				
names   = fieldnames(header);
nrnames = numel(names);

try
    fseek(fid,0,'bof');
    for r = 1:nrnames ;
        headertext = fgetl(fid);
        [headertext, headernum] = strtok(headertext,' ');
        I = cellfun(@(x,y) strcmpi(x(1:4),y(1:4)),names,repmat({headertext},nrnames,1));
        header.(names{I}) = str2double(headernum);
    end
catch ME1
    error('header can not be read')
end


% read raster data
Z = fscanf(fid,'%lg',[header.ncols header.nrows]);
fclose(fid);
Z(Z==header.nodata) = NaN;
Z = Z';
% create X and Y using meshgrid
refmat = [0 -header.cellsize;...
          header.cellsize 0;...
          header.xllcorner+(0.5*header.cellsize) - header.cellsize  ...
          (header.yllcorner+(0.5*header.cellsize))+((header.nrows)*header.cellsize)];

end

    
    
    

    
    