function ht = surf(DEM,varargin)

% surface plot for GRIDobj
%
% Syntax
%
%     surf(DEM)
%     surf(DEM,A)
%     surf(...,pn,pv,...)
%     h = ...
%
% Description
%
%     surf for GRIDobj overloads the surf command and thus provides fast
%     access to 3D visualization of digital elevation models. Note that
%     GRIDobj/surf automatically resamples the DEM so that the maximum of
%     rows or columns does not exceed 500. This ensures that the surface
%     is efficiently drawn.
%
% Input arguments
%
%     DEM          digital elevation model (GRIDobj)
%     A            grid to define color (GRIDobj) 
%
% Parameter name/values
%
%     'exaggerate'   height exaggeration, default = 1
%      
%     and all property name/value pairs allowed by surface objects.
% 
% 
%
% Author: Wolfgang Schwanghart (w.schwanghart[at]geo.uni-potsdam.de)
% Date: 30. January, 2013


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% change threshold (maximum number of rows or cols) here 
maxrowsorcols = 1000;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if max(DEM.size)>maxrowsorcols;
    resamplefactor = maxrowsorcols/max(DEM.size);
    
    DEM = resample(DEM,resamplefactor);
    
    if ~isempty(varargin) && isa(varargin{1},'GRIDobj');
        A = varargin{1};
        varargin(1) = [];
        A = resample(A,DEM);
        overlay = true;
    else
        overlay = false;
    end
    
%     warning('TopoToolbox:surf',...
%         ['DEM size exceeds maximum default number of rows or columns of ' num2str(maxrowsorcols) '\n'...
%          'and has been resampled.']);
        
    
else
    if ~isempty(varargin) && isa(varargin{1},'GRIDobj');
        A = varargin{1};
        varargin(1) = [];
        validatealignment(DEM,A);
        overlay = true;
    else
        overlay = false;
    end
end

% go through varargin to find 'exaggerate'
TF = strcmpi('exaggerate',varargin);
ix = find(TF);
if ~isempty(ix)
    exagg = varargin{ix+1};
    varargin([ix ix+1]) = [];
else
    exagg = 1;
end

[x,y] = refmat2XY(DEM.refmat,DEM.size);    

if overlay
    h = surf(x,y,double(DEM.Z),double(A.Z),varargin{:});
else
    h = surf(x,y,double(DEM.Z),varargin{:});
end

exaggerate(gca,exagg);
shading interp
camlight

    
if nargout == 1
    ht = h;
end


function exaggerate(axes_handle,exagfactor)

% elevation exaggeration in a 3D surface plot
%
% Syntax
%
%     exaggerate(axes_handle,exagfactor)
%
% Description
%
%     exaggerate is a simple wrapper for calling set(gca...). It controls
%     the data aspect ratio in a 3D plot and enables elevation
%     exaggeration.   
%
% Input
%
%     axes_handle   digital elevation model
%     exagfactor    exaggeration factor (default = 1)
%
% Example
%
%     load exampledem
%     for r = 1:4;
%     subplot(2,2,r);
%     surf(X,Y,dem); exaggerate(gca,r);
%     title(['exaggeration factor = ' num2str(r)]);
%     end
%
% 
% See also: SURF
%
%
% Author: Wolfgang Schwanghart (w.schwanghart[at]unibas.ch)
% Date: 6. September, 2010

if nargin == 1;
    exagfactor = 1;
end
axis(axes_handle,'image');
set(axes_handle,'DataAspectRatio',[1 1 1/exagfactor]);



