function [GS,x,y] = STREAMobj2mapstruct(S)

% convert instance of STREAMobj to mapstruct
%
% Syntax
%
%     MS = STREAMobj2mapstruct(S)
%     [MS,x,y] = STREAMobj2mapstruct(S)
%
% Description
%
%     A mapstruct is a structure array that contains vector geographic
%     data. STREAMobj2mapstruct converts an instance of STREAMobj to a
%     mapstruct MS. MS can be exported to a shapefile using the function
%     shapewrite available with the Mapping Toolbox.
%
%     MS contains individual line features for each river reach with unique
%     streamorder and contains following fields:
%
%        Geometry: 'Line'
%               X: vector with vertices of x-coordinates
%               Y: vector with vertices of y-coordinates
%     streamorder: stream order (Strahler)
%              IX: stream id
%        tribtoIX: tributary to stream with id IX
%
%     The additional output arguments x and y are nan-separated vectors
%     that can be used to plot the line features.
%
% Input arguments
%
%     S       STREAMobj
%     
% Output arguments
%
%     MS      mapstruct
%     x       x coordinate vector
%     y       y coordinate vector
%
%
% See also: shapewrite
%
%
% Author: Wolfgang Schwanghart (w.schwanghart[at]geo.uni-potsdam.de)
% Date: 23. April, 2013


% get streamorder
s = streamorder(S);
% get channelheads
% create sparse matrix to obtain channelheads, confluences and outlets
M = sparse(double(S.ix),double(S.ixc),true,numel(S.x),numel(S.x));
% results are logical vectors
outlets = (sum(M,2) == 0) & (sum(M,1)' > 0);


% channel index counter
chc = 1;
CHN = zeros(size(outlets));

% create empty mapstruct
GS = struct('Geometry',{},...
            'X',{},...
            'Y',{},...
            'streamorder',{},...
            'IX',{},...
            'tribtoIX',{});
            

for r = numel(S.ix):-1:1;
    
    if CHN(S.ixc(r)) ==  0;
        CHN(S.ixc(r)) = chc;
        GS(CHN(S.ixc(r))).Geometry = 'Line';
        GS(CHN(S.ixc(r))).X = S.x(S.ixc(r));
        GS(CHN(S.ixc(r))).Y = S.y(S.ixc(r));
        GS(CHN(S.ixc(r))).streamorder = s(S.ixc(r));
        GS(CHN(S.ixc(r))).IX = chc;
        
        chc = chc+1;
    end
    
    if s(S.ixc(r)) == s(S.ix(r))
        CHN(S.ix(r)) = CHN(S.ixc(r));
    
        GS(CHN(S.ix(r))).X(end+1) = S.x(S.ix(r));
        GS(CHN(S.ix(r))).Y(end+1) = S.y(S.ix(r));
      
        CHN(S.ixc(r)) = CHN(S.ix(r));
        
    elseif s(S.ixc(r)) ~= s(S.ix(r))
        
        
        CHN(S.ix(r)) = chc;
        GS(CHN(S.ix(r))).Geometry = 'Line';
        GS(CHN(S.ix(r))).X = S.x(S.ixc(r));
        GS(CHN(S.ix(r))).Y = S.y(S.ixc(r));
        
        GS(CHN(S.ix(r))).X(end+1) = S.x(S.ix(r));
        GS(CHN(S.ix(r))).Y(end+1) = S.y(S.ix(r));
        
        GS(CHN(S.ix(r))).IX = CHN(S.ix(r));
        GS(CHN(S.ix(r))).streamorder = s(S.ix(r));
        GS(CHN(S.ix(r))).tribtoIX = CHN(S.ixc(r));

        chc = chc+1;
    end
end
      
for r = 1:numel(GS);
    GS(r).X = [GS(r).X(end:-1:1) nan];
    GS(r).Y = [GS(r).Y(end:-1:1) nan];
    if isempty(GS(r).tribtoIX)
        GS(r).tribtoIX = NaN;
    end
    
end

if nargout>1
    x = [GS.X]';
    y = [GS.Y]';
end