function h = plotdzm(SW,M,varargin)
% PLOTDZM creates distance-elevation-slope angle plot of SWATHobj
%
% Syntax
%
%     plotdzm(SW,M)
%     plotdzm(SW,M,...)
%     h = ...;
%
% Description
%
%     PLOTDZM creates a profile-view plot of a SWATHobj, showing the
%     z-values as a function of distance along the profile of the SWATHobj
%     and color-coded by the z-values in the GRIDobj M.

%     plot may also be color-coded by the z-values, usually elevation, for
%     which either matlab's 'plot' or 'scatter' function can be used. This
%     needs to be specified in the parameters (see below) and affects the
%     speed of the plotting, which can be slow if the SWATHobj contains
%     many data points.
%
% Input arguments
%
%     S     instance of SWATHobj
%     M     instance of GRIDobj
%
%     Parameter name/value pairs
%
%     'left'   {true}, false
%     determines if the left half (as seen from the direction of the 
%     central line) of the SWATHobj is plotted
%
%     'right'   {true}, false
%     determines if the right half (as seen from the direction of the 
%     central line) of the SWATHobj is plotted
%
%     'distadjust'   {0}, scalar
%     allows shifting the x-axis by a scalar value. This is useful when
%     alligning SWATHobj that were obtained along, e.g., a certain reach of
%     a drainage network, with the STREAMobj it was derived from
%
%     'merge'    true,{false}
%     determines if different components of a SWATHobj will be plotted in
%     one or separate figures
%
%     'zmode'   {'absolute'}, 'relative'
%     assigns elevations along the SWATHobj either in absolute values
%     (default), or relative to the minimum value across each line in the
%     SWATHobj (relative)
%
%     'sortm',   'ascend', {'descend'}, 'none'
%     determines if and how the values within each component of a SWATHobj
%     are sorted before being plotted. This parameter affects the
%     visibility of specific populations of the data points if the markers
%     overlap in the plot.
%
%     'colormap'   {'jet'}, one of Matlab's builtin colormaps
%     determines the colors used for plotting the data points if 'colorz'
%     is set to true. Note that if plotting on top of, e.g., a DEM produced
%     by imagesc(DEM), the colorbar of the DEM will change accoringly.
%
%     'colormode'   {'normal'}, 'inverse'
%     determines how the colors change with values. Default is 'normal',
%     i.e., low (high) z-values correspond to blue (red) colors in the
%     'jet' colormap.
%
%     'colorrange' 1x2 scalar vector
%     determines the minimum and maximum values of the colormap range.
%     Default values are the minimum and maxmum values of the entire
%     SWATHobj.
%
%     'colorbar', {true}, false
%     includes a colorbar
%
%     'plotmode'   {'plot'}, 'scatter'
%     switches between Matlab's 'plot' and 'scatter' function for making
%     the plot. The scatter command is uduslly faster when 'colorz' is set
%     to true
%
%     'markersize'   {2}, scalar
%     size of the circular markers used to plot the location of the data 
%     points
%
%
% Output arguments (optional)
%
%     h    handle object of the plotted features. If more than one feature 
%          (trace, outline, points) is plotted, h contains more than one 
%          handle
%
% Examples
%
%     DEM = GRIDobj('srtm_bigtujunga30m_utm11.tif');
%     FD = FLOWobj(DEM,'preprocess','carve');
%     FA = flowacc(FD);
%     S = STREAMobj(FD,FA>1e6/FA.cellsize/FA.cellsize);
%     S = klargestconncomps(S);
%     S = trunk(S);
%     SW = SWATHobj(DEM,S,'dx',100,'dy',100,'width',2000,'smooth',11,'plot',false);
%     imagesc(DEM), hold on, plot(SW)
%     G = gradient8(DEM,'degree');
%     plotdzm(SW,G,'colorrange',[0 35]); 
%
%
% Author: Dirk Scherler (scherler[at]caltech.edu)
% Date: June, 2013


cmaps = {'bone','colorcube','cool','copper','flag',...
    'gray','hot','hsv','jet','lines','pink','prism',...
    'spring','summer','white','winter'};

% Parse inputs
p = inputParser;
p.FunctionName = 'plotdzdz';
addRequired(p,'SW',@(x) isa(x,'SWATHobj'));
addRequired(p,'M',@(x) isa(x,'GRIDobj'));
addParamValue(p,'left',true,@(x) islogical(x))
addParamValue(p,'right',true,@(x) islogical(x))
addParamValue(p,'distadjust',0,@(x) isnumeric(x))
addParamValue(p,'merge',false,@(x) islogical(x))
addParamValue(p,'zmode','absolute',@(x) ismember(x,{'absolute','relative'}))
addParamValue(p,'colormap','jet',@(x) ismember(x,cmaps))
addParamValue(p,'colormode','normal',@(x) ismember(x,{'normal','inverse'}))
addParamValue(p,'colorrange',[-inf,inf],@(x) isnumeric(x))
addParamValue(p,'colorbar',true, @(x) islogical(x))
addParamValue(p,'sortm','descend',@(x) ismember(x,{'ascend','descend','none'}))
addParamValue(p,'plotmode','scatter',@(x) ismember(x,{'plot','scatter'}))
addParamValue(p,'markersize',4,@(x) isnumeric(x))

parse(p,SW,M,varargin{:});

% parameters
distadjust = p.Results.distadjust;
left       = p.Results.left;
right      = p.Results.right;
merge      = p.Results.merge;
zmode      = p.Results.zmode;
colmap     = p.Results.colormap;
colmode    = p.Results.colormode;
colrange   = p.Results.colorrange;
colbar     = p.Results.colorbar;
sortm      = p.Results.sortm;
plotmode   = p.Results.plotmode;
markersize = p.Results.markersize;


[SWG] = mapswath(SW,M);
    
if ~left && ~right
    warning('Empty SWATHobj: nothing to plot')
    return;
end

cmap = colormap(colmap)';
if strcmp(colmode,'inverse'); cmap = flipud(cmap); end
minz = colrange(1);
maxz = colrange(2);
if isinf(minz); minz = min(min(cell2mat(SWG.Z))); end
if isinf(maxz); maxz = max(max(cell2mat(SWG.Z))); end
CX = linspace(minz,maxz,length(cmap));

for i = 1 : length(SW.xy0)
    
    if ~merge; figure; end
    
    if ~left
        ny = ceil(length(SW.disty{i})/2)+1;
        SW.Z{i} = SW.Z{i}(ny:end,:);
        SWG.Z{i} = SWG.Z{i}(ny:end,:);
    elseif ~right
        ny = floor(length(SW.disty{i})/2);
        SW.Z{i} = SW.Z{i}(1:ny,:);
        SWG.Z{i} = SWG.Z{i}(1:ny,:);
    end
    
    if strcmp(zmode,'relative')
        minz = min(SW.Z{i},[],1);
        SW.Z{i} = SW.Z{i}-repmat(minz,length(SW.disty{i}),1);
    end

    [zr,~] = size(SW.Z{i});
    d = repmat(SW.distx{i}',zr,1);
    z = reshape(SW.Z{i},numel(SW.Z{i}),1);
    s = reshape(SWG.Z{i},numel(SWG.Z{i}),1);
    d = reshape(d,numel(d),1);
    
    % Sort by slope
    if ismember(sortm,{'ascend','descend'})
        [s,ix] = sort(s,sortm);
        z = z(ix);
        d = d(ix);
    end
        
    % Interpolate colors
    mfcol = [interp1(CX,cmap(1,:),s),...
        interp1(CX,cmap(2,:),s),interp1(CX,cmap(3,:),s)];
    
    switch plotmode
        case 'plot'
            for k = 1 : length(z)
                plot(d(k)+distadjust,z(k),'s','color',mfcol(k,:),'Markerfacecolor',mfcol(k,:),'Markersize',markersize), hold on
            end
        case 'scatter'
            scatter(d+distadjust,z,markersize,mfcol,'square','filled')     
    end
    
end % loop over SWATHobj entries
drawnow
xlabel(sprintf('Distance along profile (%s)',SW.xyunit))
ylabel(sprintf('Z (%s)',SW.zunit))

if colbar
    hc = colorbar;
    yt = get(hc,'YTick');
    set(hc,'YTickLabel',(yt.*(maxz-minz)+minz)');
end

if nargout==1;
    varargout = h;
end

