function OUT = tidyswath(SW)

% TIDYSWATH removes overlapping points from SWATHobj
%
% Syntax
%
%     OUT = tidyswath(SW)
%
% Description
%
%     TIDYSWATH uses the function 'lineSegmentIntersect' by U. Murat Erdem,
%     freely available on Matlab Central, to remove overlapping points in a
%     SWATHobj by setting the corresponding Z values to NaN.
%
% Input arguments
%
%     SW      instance of SWATHobj
%
% Output arguments
%
%     OUT    instance of SWATHobj
%
% Examples
%
%     DEM = GRIDobj('srtm_bigtujunga30m_utm11.tif');
%     FD = FLOWobj(DEM,'preprocess','carve');
%     FA = flowacc(FD);
%     S = STREAMobj(FD,FA>1e6/FA.cellsize/FA.cellsize);
%     S = trunk(klargestconncomps(S));
%     SW = SWATHobj(DEM,S,'dx',100,'dy',100,'width',3000,'smooth',11,'plot',false);
%     SW2 = tidyswath(SW);
%     plot(SW2)
%
%
% Author: Dirk Scherler (scherler[at]caltech.edu)
% Date: June, 2013


if isa(SW,'SWATHobj')
    
    % remove duplicates from mapping
    if ~exist('lineSegmentIntersect','file')
        error({'TIDYSWATH: Removing duplicates from SWATHobj requires the function';...
            '"lineSegmentIntersect" by U. Murat Erdem.';...
            'Available on Matlab Central'});
    else
        
        OUT = SW;
        
        for k = 1 : length(SW.xy0)
            [ny,~] = size(SW.X{k});
            if mod(ny,2)==0; ctr = ny/2; xtr = 1;
            else ctr = (ny+1)/2; xtr = 0; end

            for sides = 1 : 2
                if sides==1; edge = 1;
                else edge = ny; ctr = ctr+1+xtr; end
                if edge < ctr; step = -1; else step = 1; end
                id = ctr:step:edge;
                d = abs(SW.disty{k}(id));
                maxd = max(d);

                % the points near the center of the swath are defined as the starting points
                x1 = SW.X{k}(ctr,:)';
                y1 = SW.Y{k}(ctr,:)';
                % the points on the edge of the swath are defined as the ending points
                x2 = SW.X{k}(edge,:)';
                y2 = SW.Y{k}(edge,:)';
                XY1 = [x1,y1,x2,y2];
                XY1(isnan(XY1(:,1)),:) = [];
                XY2 = XY1;
                LSI = lineSegmentIntersect(XY1,XY2);
                U = triu(LSI.intAdjacencyMatrix);

                % check each intersection
                [nru,ncu] = size(U);
                IX = find(U);
                for i = 1 : length(IX)
                    d1 = abs( LSI.intNormalizedDistance1To2(IX(i)) );
                    d2 = abs( LSI.intNormalizedDistance2To1(IX(i)) );
                    [p1,p2] = ind2sub([nru,ncu],IX(i));
                    % determine which line to cut short
                    if d1 <= d2;
                        thisp = p2;
                        thisd = d2*maxd;
                    else
                        thisp = p1;
                        thisd = d1*maxd;
                    end
                    % exclude points along line beyond intersection
                    ix = d>thisd;
                    OUT.Z{k}(id(ix),thisp) = nan;
                    %OUT.X{k}(id(ix),thisp) = nan;
                    %OUT.Y{k}(id(ix),thisp) = nan;
                end
            end
        end
    end
    
end