% TopoToolbox
% Version 2.0 (R2012a) 19-June-2013
%
% TopoToolbox provides a set of Matlab functions that support the analysis
% of relief and flow pathways in digital elevation models. The major 
% aim of TopoToolbox is to offer helpful analytical GIS utilities in a 
% non-GIS environment in order to support the simultaneous application
% of GIS-specific and other quantitative methods.
%
% If you have any questions or remarks, please contact the authors:
%
% Wolfgang Schwanghart
% w.schwanghart[at]geo.uni-potsdam.de
%
% Dirk Scherler
% scherler[at]caltech.edu
% 
%
% Objects
%
%     GRIDobj         - object for gridded, geospatial data
%     FLOWobj         - object for flow direction
%     STREAMobj       - object for stream (channel) networks
%
% Graphical user interfaces
%
%     flowpathapp     - Map, visualize and export flowpaths that start at manually set channelheads                     
%     preprocessapp   - interactive tool for hydrological conditioning
%     slopeareatool   - Interactively create slope area plots and fit power laws                    
%     topoapp         - Create instance of a topoapp
%
% GRIDobj methods
%
%     GRIDobj           - Create instance of a GRIDobj
%     GRIDobj2ascii     - write/export GRIDobj to ESRI ArcGIS ASCII file
%     GRIDobj2geotiff   - Exports an instance of GRIDobj to a geotiff file
%     GRIDobj2mat       - convert GRIDobj to matrix and coordinate vectors
%     acv               - Anisotropic coefficient of variation (ACV) 
%     aspect            - aspect (angle of exposition) from a digital elevation model (GRIDobj)
%     castshadow        - cast shadow
%     coord2ind         - convert x and y coordinates to linear index
%     coord2sub         - convert x and y coordinates to subscripts into a GRIDobj
%     crop              - crop an instance of GRIDobj with axis-aligned minimum bounding box
%     curvature         - 8-connected neighborhood curvature of a digital elevation model 
%     dilate            - morphological dilation
%     elevateminima     - elevate regional minima in a DEM to their lowest neighbor
%     erode             - morphological erosion
%     fillsinks         - fill/remove pits, sinks or topographic depressions
%     filter            - edge detection using sobel filter 
%     getcoordinates    - get coordinate vectors of an instance of GRIDobj
%     gradient8         - 8-connected neighborhood gradient and aspect of a digital elevation model
%     hillshade         - create hillshading from a digital elevation model (GRIDobj)
%     hypscurve         - plot hypsometric curve of a digital elevation model
%     identifyflats     - identify flat terrain in a digital elevation model
%     imagesc           - Scale data in GRIDobj and display as image object
%     imageschs         - plot hillshade image with overlay
%     ind2coord         - convert linear index to x and y coordinates
%     info              - detailed information on GRIDobj instance
%     inpaintnans       - interpolate missing values in a grid (GRIDobj)
%     interp            - interpolate to query locations
%     isnan             - returns array elements that are NaNs as logical grid
%     localtopography   - Local topography
%     postprocflats     - postprocess flat terrain for visualization purpose
%     reclassify        - generate univariate class intervals for an instance of GRIDobj
%     resample          - resample grid to alter spatial resolution
%     roughness         - terrain ruggedness, position and roughness indices of DEMs
%     shufflelabel      - shufflelabel randomly relabels a label matrix
%     snap2stream       - snap gauges or pour points to stream raster
%     sub2coord         - convert subscripts to x and y coordinates
%     surf              - surface plot for GRIDobj
%     validatealignment - validates whether instances of GRIDobj are spatially aligned  
% 
% FLOWobj methods
% 
%     FLOWobj             - Create flow direction object
%     FLOWobj2GRIDobj     - create ESRI ArcGIS flow direction grid from FLOWobj
%     FLOWobj2M           - convert instance of FLOWobj to flow direction matrix 
%     FLOWobj2gradient    - gradient along flow direction
%     dependencemap       - downslope area for specific locations in a digital elevation model
%     drainagebasins      - drainage basin delineation/catchments
%     find                - find indices and values of edges in the flow direction graph
%     flowacc             - flow accumulation (upslope area, contributing area)
%     flowconvergence     - compute flow convergence of a digital elevation model
%     flowdistance        - flow distance in upstream and downstream direction
%     flowpathextract     - extract linear indices of a single flowpath in a DEM
%     imposemin           - minima imposition (carving) along drainage network
%     ind2coord           - convert linear index to x and y coordinates
%     influencemap        - downslope area for specific locations in a digital elevation model
%     ismulti             - check if FD is multi or single flow direction
%     saveobj             - Create flow direction object
%     streamorder         - calculate Strahler Stream Order Grid from FLOWobj
%     streampoi           - stream points of interest
%     upslopestats        - upslope statistics of a variable based on the flow direction matrix
%     validatealignment   - validates whether instances of FLOWobj and GRIDobj are spatially aligned
%     vertdistance2stream - vertical distance to streams
% 
% STREAMobj methods
% 
%     STREAMobj           - Create stream object (STREAMobj)
%     STREAMobj2GRIDobj   - convert an instance of STREAMobj to an instance of GRIDobj
%     STREAMobj2XY        - convert instance of STREAMobj to NaN-separated X and Y coordinates
%     STREAMobj2cell      - convert instance of STREAMobj to cell array of stream objects
%     STREAMobj2mapstruct - convert instance of STREAMobj to mapstruct
%     extractconncomps    - interactive stream network selection
%     gradient            - stream gradient
%     klargestconncomps   - retain k largest connected components in an instance of STREAMobj
%     modify              - modify instance of STREAMobj to meet user-defined criteria
%     plot                - plot instance of STREAMobj
%     plotdz              - plot upstream distance version elevation of a stream network
%     removeshortstreams  - Remove first order streams with a length less than specified
%     snap2stream         - snap locations to nearest stream location
%     streamorder         - calculate Strahler Stream Order from STREAMobj
%     trunk               - extract trunk stream (longest stream) 
%     validatealignment   - validates whether instances of STREAMobj and GRIDobj are spatially aligned
% 
% SWATHobj methods
% 
%     SWATHobj     - Create swath profile object (SWATHobj)
%     mapswath     - MAPSWATH obtains Z values along swath from arbitrary GRIDobj
%     modify       - MODIFY sets fields in SWATHobj to NaN according to several optional rules
%     plot         - plot instance of SWATHobj
%     plotdz       - PLOTDZ creates distance-elevation plot of SWATHobj
%     plotdzm      - PLOTDZM creates distance-elevation-slope angle plot of SWATHobj
%     plotz        - PLOTZ creates plot of statistics calculated tranverse to SW(i)ath profile
%     swath2latlon - SWATH2LATLON converts spatial fields in SWATHobj to lat,lon
%     tidy         - TIDY removes overlapping points from SWATHobj
%     tidyswath    - TIDYSWATH removes overlapping points from SWATHobj
%
% Other tools
%
%     compilemexfiles - compile mex-functions that come with TopoToolbox 2
%     coord2ind       - convert xy coordinates to linear index
%     exaggerate      - elevation exaggeration in a 3D surface plot
%     getdistance     - cumulative distance along path defined by vertice coordinates
%     getextent       - get current axis extent
%     interpline      - distribute vertices evenly along line with irregularly spaced vertices
%     ixneighbors     - neighbor indexing in a matrix
%     label2poly      - plot region outlines with polyline
%     landcolor       - colormap for the display of DEMs
%     setextent       - set current axis extent    
%     showmethods     - displays class method names and H1 lines in the command line
%
% Documentation
%
%   usersguide_1_intro      - introductory script to the toolbox
%   usersguide_2_mfd        - how to calculate Multiple Flow Direction in
%                             TopoToolbox 2
%
%
%   ---------------------------------------------------------------------
%     TopoToolbox is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

